@ECHO off

REM Declare script variable

SET CONFIRMED=
SET RESULT=0
SET FULLINTERACTIVE=No

REM There are no any settings passed from the command line: treat the installation as a full
REM interactive and don't close the console window on the exit.
IF [%1]==[] SET FULLINTERACTIVE=Yes


REM Declare settings

SET SPECIAL_SETTINGS=DEFAULT SILENT SMTPCHECK INSTANCENAME
SET OPTIONAL_SETTINGS=TARGETDIR DBSERVER DBNAME DBLOGIN DBPASSWORD SITENAME SITEPOOL SITEUSER DOMAINSETUP SITEIPADDRESS SITEPORT
SET MANDATORY_SETTINGS=SMTPHOST SMTPFROM SMTPUSER SMTPPASSWORD CONTROLLERCERT PROVIDERLOGIN PROVIDERPASSWORD
SET INTERACTIVE_SETTINGS=%OPTIONAL_SETTINGS% %MANDATORY_SETTINGS%
SET ALL_SETTINGS=%SPECIAL_SETTINGS% %INTERACTIVE_SETTINGS%

FOR %%x in (%ALL_SETTINGS%) DO (
	SET %%x=
)


Rem Define settings

SET SMTPCHECK=Yes

REM Read command line arguments
:While1
IF [%1]==[] GOTO :EndWhile1
	SET name=%~1

	SET list1="%SPECIAL_SETTINGS: =""%"
	CALL SET check1=%%list1:"%name%"=%%

	SET list2="%INTERACTIVE_SETTINGS: =""%"
	CALL SET check2=%%list2:"%name%"=%%

	REM All special settings have to be unquoted
	IF NOT [%list1%]==[%check1%] (
		CALL SET %%name%%=%%~2
	) ELSE IF NOT [%list2%]==[%check2%] (
		REM All interactive settings have to be quoted
		SET %name%="%~2"
	) ELSE (
		REM Unknown setting
		ECHO Error: Setting "%name%" is unknown
		SET RESULT=-1
		GOTO :STOP
	)
	SHIFT /1
	SHIFT /1
	GOTO :While1
:EndWhile1


REM Declare default values

SET DEFAULT_TARGETDIR=%ProgramFiles%\Odin\Azure CSP
SET DEFAULT_DBSERVER=(local)
SET DEFAULT_DBNAME=csp
SET DEFAULT_DBLOGIN=sa
SET DEFAULT_DBPASSWORD=
SET DEFAULT_SITENAME=Azure Cloud Solution Provider - Internal
SET DEFAULT_SITEPOOL=AzureCSPInternal
SET DEFAULT_SITEUSER=AzureCSPInternalUser
SET DEFAULT_DOMAINSETUP=Yes
SET DEFAULT_SITEIPADDRESS=
SET DEFAULT_SITEPORT=
SET DEFAULT_CONTROLLERCERT=
SET DEFAULT_SMTPHOST=
SET DEFAULT_SMTPFROM=
SET DEFAULT_SMTPUSER=
SET DEFAULT_SMTPPASSWORD=
SET DEFAULT_PROVIDERLOGIN=
SET DEFAULT_PROVIDERPASSWORD=

IF EXIST "%CD%\controller.pem" SET DEFAULT_CONTROLLERCERT=%CD%\controller.pem

IF /I [%USERDOMAIN%]==[%COMPUTERNAME%] SET DEFAULT_DOMAINSETUP=No

SET CAPTION_TARGETDIR=Target directory
SET CAPTION_DBSERVER=Database server instance
SET CAPTION_DBNAME=Database name
SET CAPTION_DBLOGIN=Database administrator login
SET CAPTION_DBPASSWORD=Database administrator password
SET CAPTION_SITENAME=Web-site name
SET CAPTION_SITEPOOL=Web-site application pool
SET CAPTION_SITEUSER=Web-site user
SET CAPTION_DOMAINSETUP=Web-site user is a domain user
SET CAPTION_SITEIPADDRESS=Web-site IP address (leave empty to choose automatically the first address that fits requirements)
SET CAPTION_SITEPORT=Web-site port (leave empty to choose automatically the port that fits requirements)
SET CAPTION_PROVIDERLOGIN=Provider Azure login
SET CAPTION_PROVIDERPASSWORD=Provider Azure password
SET CAPTION_CONTROLLERCERT=APS Controller Certificate
SET CAPTION_SMTPHOST=SMTP server which can be used to send email notifications
SET CAPTION_SMTPFROM=Email address that email notifications will be sent from
SET CAPTION_SMTPUSER=SMTP user login
SET CAPTION_SMTPPASSWORD=SMTP user password


FOR %%x in (%INTERACTIVE_SETTINGS%) DO (
	CALL SET PROMPT_%%x="%%CAPTION_%%x%%"
)

SET PROMPT_DBLOGIN="%CAPTION_DBLOGIN% (leave the default value if you are going to use the integrated security)"
SET PROMPT_DBPASSWORD="%CAPTION_DBPASSWORD% (leave empty if you are going to use the integrated security)"
SET PROMPT_DOMAINSETUP="%CAPTION_DOMAINSETUP% (specify 'No' if you want to create a local user)"

IF /I [%SILENT%]==[Yes] (
	SET DEFAULT=Yes
	SET CONFIRMED=Yes
) ELSE (
	IF [%INSTANCENAME%]==[] (
		SET /P INSTANCENAME="Specify the instance name or leave empty to install the default instance: "
	)
)

IF NOT [%INSTANCENAME%]==[] (
	SETLOCAL ENABLEDELAYEDEXPANSION

	CALL :CheckStringIsSafe "%INSTANCENAME%"

	IF [!ERRORLEVEL!]==[1] (
		ECHO Error: Instance name can contain only latin symbols, digits and symbol '-'
		ENDLOCAL
		SET RESULT=-2
		GOTO :STOP
	)

	ENDLOCAL

	SET DEFAULT_TARGETDIR=%DEFAULT_TARGETDIR% ^(%INSTANCENAME%^)
	SET DEFAULT_DBNAME=%DEFAULT_DBNAME%-%INSTANCENAME%
	SET DEFAULT_SITENAME=%DEFAULT_SITENAME% ^(%INSTANCENAME%^)
)

IF [%DEFAULT%]==[] SET /P DEFAULT="Do you want to use the default installation settings [Yes/No]? "

IF /I [%DEFAULT%]==[Yes] (
	FOR %%x in (%OPTIONAL_SETTINGS%) DO (
		CALL :SetDefault %%x
	)
	Rem They must not be empty
	FOR %%x in (%MANDATORY_SETTINGS%) DO (
		IF [%%x]==[CONTROLLERCERT] (
			IF [%DEFAULT_CONTROLLERCERT%]==[] (
				CALL :ReadValue %%x
			) ELSE (
				CALL :SetDefault %%x
			)
		) ELSE (
			CALL :ReadValue %%x
		)
	)
) ELSE (
	FOR %%x in (%INTERACTIVE_SETTINGS%) DO (
		CALL :ReadValue %%x
	)
)

ECHO Installation settings
FOR %%x in (%INTERACTIVE_SETTINGS%) DO (
	call :ShowValue %%x
)

IF [%CONFIRMED%]==[] (
	SET /P CONFIRMED="Is everything correct [Yes/No]? "
)

IF /I NOT [%CONFIRMED%]==[Yes] (
	ECHO Error: Installation aborted!
	SET RESULT=-3
	GOTO :STOP
)

CALL :DeployFiles %TARGETDIR% %SILENT%
SET RESULT=%ERRORLEVEL%
IF NOT [%RESULT%]==[0] GOTO :STOP

Rem If we don't disable Delayed Expansion it won't be possible to pass '!' symbol in argument strings
SET local_setup=$True
IF /I [%DOMAINSETUP%]==["Yes"] SET local_setup=$False

SET validate_smtp=$False
IF /I [%SMTPCHECK%]==[Yes] SET validate_smtp=$True

SET COMPLUS_ApplicationMigrationRuntimeActivationConfigPath=%TARGETDIR%\deployment
PUSHD %TARGETDIR%\deployment
%windir%\System32\WindowsPowerShell\v1.0\powershell.exe -ExecutionPolicy Bypass -NoLogo ^
	-Command .\Setup.ps1 ^
	-TargetDir %TARGETDIR:"='% ^
        -DbServer %DBSERVER:"='% ^
        -DbName %DBNAME:"='% ^
        -DbLogin %DBLOGIN:"='% ^
        -DbPassword %DBPASSWORD:"='% ^
        -SiteName %SITENAME:"='% ^
        -SiteIpAddress %SITEIPADDRESS:"='% ^
        -SitePort %SITEPORT:"='% ^
        -SitePool %SITEPOOL:"='% ^
        -SiteUser %SITEUSER:"='% ^
        -LocalUser:%local_setup% ^
        -ProviderLogin %PROVIDERLOGIN:"='% ^
        -ProviderPassword %PROVIDERPASSWORD:"='% ^
        -ControllerCert %CONTROLLERCERT:"='% ^
        -SmtpHost %SMTPHOST:"='% ^
        -SmtpFrom %SMTPFROM:"='% ^
        -SmtpUser %SMTPUSER:"='% ^
        -SmtpPassword %SMTPPASSWORD:"='% ^
        -ValidateSmtp:%validate_smtp%
Rem Fix colorizer in Sublime"

SET RESULT=%ERRORLEVEL%
POPD
:STOP
IF /I [%FULLINTERACTIVE%]==[Yes] PAUSE
EXIT /B %RESULT%

REM PROCEDURE: SetDefault(var_name)
:SetDefault
	SETLOCAL ENABLEDELAYEDEXPANSION
	SET val=!%1!
	IF [%val%]==[] SET val="!DEFAULT_%1!"
	ENDLOCAL&SET %1=%val%
EXIT /B 0

REM PROCEDURE: ReadValue(var_name)
:ReadValue
	SETLOCAL DISABLEDELAYEDEXPANSION

	SET name=%1
	CALL SET value=%%%name%%%
	CALL SET message=%%PROMPT_%name%%%
	CALL SET default="%%DEFAULT_%name%%%"

	IF [%value%]==[] (
		IF [%name%]==[SMTPUSER] (
			CALL :GetValue %message% %SMTPFROM%
		) ELSE (
			CALL :GetValue %message% %default%
		)
	)

	ENDLOCAL&SET %1=%value%
EXIT /B 0

REM PROCEDURE: GetValue(prompt_string, default_value)
:GetValue
	SET value=%~2
	IF NOT "%~2"=="" (
		SET /P value="%~1 [%~2]: "
	) ELSE (
		SET /P value="%~1: "
	)
	SET value="%value%"
EXIT /B 0

REM PROCEDURE: ShowValue(var_name)
:ShowValue
	SETLOCAL ENABLEDELAYEDEXPANSION
	
	IF [%1]==[SITEUSER] (
		IF /I [%DOMAINSETUP%]==["Yes"] (
			CALL :ShowComplexValue "!CAPTION_%1!" "%USERDOMAIN%\" !%1!
		) ELSE (
			CALL :ShowComplexValue "!CAPTION_%1!" "%COMPUTERNAME%\" !%1!
		)
	) ELSE IF [%1]==[DOMAINSETUP] (
		Rem Skip it
	) ELSE IF [%1]==[DBPASSWORD] (
		IF [%DBPASSWORD%]==[""] (
			ECHO.	Database integrated security: Yes
		) ELSE (
			ECHO.	!CAPTION_%1!: !%1!
		)
	) ELSE IF [%1]==[DBLOGIN] (
		IF NOT [%DBPASSWORD%]==[""] ECHO.	!CAPTION_%1!: !%1!
	) ELSE ECHO.	!CAPTION_%1!: !%1!

	ENDLOCAL
EXIT /B 0

REM PROCEDURE: ShowComplexValue(caption, prefix, suffix)
:ShowComplexValue
	ECHO.	%~1: "%~2%~3"
EXIT /B 0

REM PROCEDURE: DeployFiles(base_dir, silent)
:DeployFiles
	SETLOCAL ENABLEDELAYEDEXPANSION

	SET base_dir=%~1
	SET silent=%2
	SET result=0

	IF EXIST "%base_dir%\deployment" (
		IF /I [%silent%]==[Yes] (
			ECHO Error: The application is already extracted into "%base_dir%". Installation cannot be continue in the silent mode.
			SET result=-4
			GOTO :ReturnDeployFiles
		)

		SET ASK=
		SET /P ASK=The application is already extracted into "%base_dir%". Do you want to continue without replacing of an old content [Yes/No]? 
		IF /I NOT [!ASK!]==[Yes] (
			ECHO Error: Installation aborted!
			SET result=-5
			GOTO :ReturnDeployFiles
		)

		SET result=0
		GOTO :ReturnDeployFiles
	)

	ECHO Unpack distributive into %base_dir%
	call :UnpackData "%base_dir%" "%base_dir%\tmp-64B0E018-4351-47EF"
	SET result=%ERRORLEVEL%

	:ReturnDeployFiles
	ENDLOCAL&SET RESULT=%result%
EXIT /B %RESULT%

REM PROCEDURE: UnpackData(base_dir, tmp_dir)
:UnpackData
	SET base_dir=%~1
	SET tmp_dir=%~2
	IF NOT EXIST "%tmp_dir%" MKDIR "%tmp_dir%"
	IF ERRORLEVEL 1 EXIT /B %ERRORLEVEL%

	ECHO ^<?xml version=^"1.0^" encoding=^"utf-8^" ?^>^<configuration^>^<startup useLegacyV2RuntimeActivationPolicy=^"true^"^>^<supportedRuntime version=^"v4.0^"/^>^</startup^>^</configuration^>> "%tmp_dir%\powershell.exe.activation_config"

	Rem Unzip 'data.zip'
	SET COMPLUS_ApplicationMigrationRuntimeActivationConfigPath=%tmp_dir%
	%windir%\System32\WindowsPowerShell\v1.0\powershell.exe ^
		-ExecutionPolicy Bypass ^
		-NoLogo ^
		-Command "Add-Type -assembly 'system.io.compression.filesystem'; [io.compression.zipfile]::ExtractToDirectory('%~dp0data.zip', '%base_dir%')"
	SET RESULT=%ERRORLEVEL%
	RMDIR /S /Q "%tmp_dir%"
EXIT /B %RESULT%

REM PROCEDURE: CheckStringIsSafe(name)
:CheckStringIsSafe
	SET str="%~1"
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:A=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:B=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:C=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:D=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:E=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:F=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:G=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:H=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:I=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:J=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:K=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:L=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:M=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:N=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:O=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:P=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:Q=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:R=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:S=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:T=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:U=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:V=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:W=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:X=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:Y=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:Z=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:a=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:b=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:c=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:d=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:e=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:f=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:g=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:h=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:i=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:j=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:k=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:l=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:m=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:n=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:o=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:p=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:q=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:r=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:s=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:t=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:u=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:v=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:w=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:x=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:y=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:z=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:0=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:1=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:2=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:3=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:4=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:5=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:6=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:7=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:8=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:9=%
	IF [%str%]==[""] EXIT /B 0
	SET str=%str:-=%
	IF [%str%]==[""] EXIT /B 0
EXIT /B 1
