﻿define([
	"aps/_Container",
	"aps/CheckBox",
	"aps/Container",
	"aps/Output",
	"aps/TextBox",
	"aps/_Widget",
	"dijit/_TemplatedMixin",
	"dojo/_base/declare",
	"dojox/mvc/at",
	"./consts.js"
], function (_Container, CheckBox, Container, Output, TextBox, _Widget, _Templated, declare, at, consts) {
	//"use strict";
	return declare([_Widget, _Container, _Templated], {
		// summary:
		// 		Widget represents an optional TextBox.


		// baseClass: [private] string
		// 		Root CSS class of the widget (ex: dijitTextBox),
		// 		used to construct CSS classes to indicate widget state.
		baseClass: "CheckTextBox",

		// isChecked: boolean
		//		Indicates whether the checkbox is selected.
		isChecked: "",

		// value: string
		//		A TextBox value.
		value: "",

		// type: string
		//		A TextBox type. Supported "number".
		//		default: null
		//		Note: cannot be changed after creation
		type: null,

		// step: float
		//		If type is set tp "number", defines default step
		//		default: 1
		//		Note: cannot be changed after creation
		step: 1,

		// title: string
		//		A text shown nearby checkbox.
		title: "",

		// minimal: float
		//		If type is set tp "number", defines minimal value
		//		default: 1
		//		Note: cannot be changed after creation
		minimal: null,

		// inPercent: boolean
		//		Indicates whether the '%' sign should be added.
		inPercent: false,

		// templateString: [private] string
		// 		Template string
		templateString: "<div data-dojo-attach-point=\"containerNode\"></div>",

		// pattern: string
		//		A TextBox pattern.
		//		default: consts.numberPattern
		//		Note: cannot be changed after creation
		pattern: consts.numberPattern,

		// invalidMessage: string
		//		A TextBox invalidMessage.
		//		default: consts.numberInvalidMessage
		//		Note: cannot be changed after creation
		invalidMessage: consts.numberInvalidMessage,

		postCreate: function() {
			// summary:
			//		Post create processing.
			// tags:
			// 		Public

			var self = this;
			self.inherited(arguments);

			self.addChild(new CheckBox({
				description: at(self, "title"),
				checked: at(self, "isChecked")
			}));

			var textbox = new TextBox({
				value: at(self, "value"),
				disabled: at(self, "isChecked").direction(at.from).transform({ format: function(val) { return !val; } }),
				required: at(self, "isChecked").direction(at.from),
				pattern: self.pattern,
				invalidMessage: self.invalidMessage
			});

			if (self.minimal !== null) {
				textbox.set("constraints", { min: self.minimal });
			}

			if (self.type === "number") {
				textbox.set("pattern", consts.numberPattern);
				textbox.set("invalidMessage", consts.numberInvalidMessage);
				textbox.set("type", self.type);
				textbox.set("step", self.step);
			}

			if (self.inPercent) {
				var container = new Container();
				container.addChild(textbox);
				container.addChild(new Output({ value: "%" }));
				self.addChild(container);
			} else {
				self.addChild(textbox);
			}
		}
	});
});