define([
		"dojo/_base/declare",
		"aps/Grid",
		"aps/DateTextBox",
		"aps/Output",
		"aps/Switch",
		"dojo/dom-construct",
		"dojo/_base/array",
		"dojo/_base/lang",
		"dojo/query"
], function(declare, Grid, DateTextBox, Output, Switch, domConstruct, arr, lang, query){
	return declare([Grid], {
		// sync with billing time options
		// 	default values: {sync: false, showSwitch: false, switchLabel: "Synchronize Usage Reports with Billing System Time"}
		syncWithBillingTime: null,

		postCreate: function(){
			var self = this;
			self.syncWithBillingTime = lang.mixin({
				sync: false,
				showSwitch: false,
				switchLabel: _("Synchronize Usage Reports with Billing System Time")
			}, self.syncWithBillingTime);

			self.inherited(arguments);
		},

		_createSearchPanel: function(root){
			var self = this,
				ul,
				node,
				frag = document.createDocumentFragment(),
				searchBoxes	 = query(".search-box", root);

			if(searchBoxes.length > 0)
				arr.forEach(searchBoxes, function(box){ domConstruct.destroy(box); });

			function submit(){
				self.set("_filters", []);
				query("input, select", self.toolbarNode).forEach(function(item){
					if(item.id && item.value)
						self._filters[item.id.split('-').pop()] = item.value;
				});
				self.selectionArray.splice(0, self.selectionArray.length);
				self._query(0, null, null, self.get("rowsPerPage"), 1);
			}

			function checkCustomDateFilter(filter){
				return filter.type && filter.type.toLowerCase() === "custom_date" && filter.start && filter.end;
			}

			arr.forEach(this.columns, function(column){
				if(column.filter){
					if(!node)
						node = query("form", domConstruct.create("div", { "class": "search-box", innerHTML: "<form></form>" }, frag))[0];

					if(!ul)
						ul = domConstruct.create("ul", {}, node);

					var caption = checkCustomDateFilter(column.filter) ? column.filter.start.title : (column.filter.title ? column.filter.title : column.name);

					var li = domConstruct.create("li", { innerHTML: "<div><span>" + caption + "</span><span></span></div>" }, ul);

					var li_body = query("span", li)[1];
					if(column.filter.options){
						var select = domConstruct.create("select", { id: self.id + "-search-select-" + column.field	}, li_body);
						domConstruct.create("option", {
							innerHTML: self.filterOptAllText,
							selected: "selected",
							value: ""
						}, select);
						arr.forEach(column.filter.options, function(option){
							var opt = domConstruct.create("option", {
								innerHTML: option.label,
								value: typeof option.value === "boolean" ? option.value + "()" : option.value
							}, select);

							if(opt.value == column.filter.value)
								opt.selected = "selected";
						});
					} else if (checkCustomDateFilter(column.filter)) {
						var start = new DateTextBox({
							id: self.id + "-search-date-start_" + column.field,
							value: column.filter.start.value,
							autoSize: true,
							missingMessage: column.filter.start.missingMessage,
							required: true,
							constraints: column.filter.start.constraints,
							setDefault: column.filter.start.setDefault,
							setNewValue: column.filter.start.setNewValue,
							validateAndShowError: column.filter.start.validateAndShowError
						});

						var end = new DateTextBox({
							id: self.id + "-search-date-end_" + column.field,
							value: column.filter.end.value,
							autoSize: true,
							missingMessage: column.filter.end.missingMessage,
							required: true,
							constraints: column.filter.end.constraints,
							setDefault: column.filter.end.setDefault,
							setNewValue: column.filter.end.setNewValue,
							validateAndShowError: column.filter.end.validateAndShowError
						});

						start.own(start.watch("value", function(name, oldVal, newVal){
							if (newVal){
								var oldContrains = end.get("constraints");
								end.set("constraints", { min: newVal, max: oldContrains.max });
								start.setNewValue(newVal);
							}
						}));
						end.watch(end.watch("value", function(name, oldVal, newVal){
							if (newVal){
								var oldContrains = start.get("constraints");
								start.set("constraints", { max: newVal, min: oldContrains.min });
								end.setNewValue(newVal);
							}
						}));

						domConstruct.place(start.domNode, li_body);

						var li_end = domConstruct.create("li", { innerHTML: "<div><span>" + column.filter.end.title + "</span><span></span></div>" }, ul);
						var li_end_body = query("span", li_end)[1];
						domConstruct.place(end.domNode, li_end_body);
					} else {
						domConstruct.create("input", {
							id: self.id + "-search-text-" + column.field,
							type: "text",
							value: column.filter.value || ""
						}, li_body);
					}
				}
			});

			if(node){
				query("input, select", node).on("keypress", function(e){
					if(e.keyCode == 13){
						submit();
						e.preventDefault();
					}
				});

				domConstruct.create("a", {
					"class": "s-btn sb-search",
					href: "javascript:void(0)",
					innerHTML: "<i><span>" + self.searchButtonText + "</span></i>",
					onclick: submit
				}, node);

				domConstruct.create("a", {
					"class": "s-btn sb-show-all",
					href: "javascript:void(0)",
					innerHTML: "<i><span>" + self.resetSearchButtonText + "</span></i>",
					onclick: function(){
						self.set({ "_filters": [] });
						self.selectionArray.splice(0, self.selectionArray.length);
						query("input, select", self.toolbarNode).forEach(function(item){
							item.value = "";
							if(item.selectedIndex) item.selectedIndex = 0;
						});
						self._query(0, null, null, self.get("rowsPerPage"));
					}
				}, node);


				if (self.syncWithBillingTime && self.syncWithBillingTime.showSwitch) {
					var switchContainer = domConstruct.create("div", {"class": "switchArea"}, node, "after");
					var switchArea = domConstruct.create("span", {"class": "active-list"}, switchContainer);
					var switchLabel = new Output({
						value: self.syncWithBillingTime.switchLabel
					});
					domConstruct.place(switchLabel.domNode, switchArea);
					var switcher = new Switch({
						status: self.syncWithBillingTime.sync ? "on" : "off",
						"class": "syncSwitch",
						onClick: function(){
							var switcher = this;
							var newVal = !self.syncWithBillingTime.sync;
							self.syncWithBillingTime.sync = newVal;
							self.syncWithBillingTime.onSwitch && self.syncWithBillingTime.onSwitch();
							self.refresh();
							switcher.set("status", newVal ? "on" : "off");
						}
					});
					domConstruct.place(switcher.domNode, switchArea);
				}
			}
			domConstruct.place(frag, root);
		},

		_setSortAttr: function(val){
			// fix empty "" descending for "reportedTime" attribute
			if (val.attribute && val.attribute === "reportedTime" && !val.descending && typeof val.descending === "string"){
				val.descending = true;
			}

			if(this.sort.attribute == val.attribute && this.sort.descending == val.descending) return;
			this._set("sort", { attribute: val.attribute || "", descending: val.descending || "" });
			this._putState({ sort: { attribute: val.attribute || "", descending: val.descending || "" } });
			if(this.get("_load"))
				this._query((this.get("currentPage")-1)*this.get("rowsPerPage"));
		}
	});
});