define([
	"dijit/registry",
	"aps/Container",
	"aps/FieldSet",
	"aps/TextBox",
	"aps/Output",
	"dojo/Deferred",
	"dojox/mvc/at",
	"dojox/mvc/getStateful",
	"dojox/mvc/getPlainValue",
	"./customization/ModalWindow.js",
	"./consts.js",
	"./utils.js",
	"aps/i18n"
], function(
	registry,
	Container,
	FieldSet,
	TextBox,
	Output,
	Deferred,
	at,
	getStateful,
	getPlainValue,
	ModalWindow,
	consts,
	utils,
	_
	){
	"use strict";

	var setMarkupPopup = {
		_result: null,

		create: function(){
			var self = this;
			self._setMarkupPopup = new ModalWindow({
				header:_("Set Markup"),
				model: getStateful({
					reseller_markup: ""
				}),
				submitButtonText: _("OK"),
				onCancel: function() {
					self._result.resolve();
					return true;
				},
				onSubmit: function() {
					var isValid = self._markupValid();
					if(!isValid){
						return false;
					}
					var newMarkup = getPlainValue(self._setMarkupPopup.model).reseller_markup;
					self._result.resolve(utils.parseStringToNumber(newMarkup));
					return true;
				}
			});

			var popupContainer = new Container({
				id:"setMarkupPopup"
			});
			var resellerMarkupFieldSet = new FieldSet({
				id: "setMarkupFieldSet"
			});

			popupContainer.addChild(resellerMarkupFieldSet);

			var markupContainer = new Container({
				id: "setMarkupContainer",
				label: _("Reseller Markup"),
				required: true
			});

			resellerMarkupFieldSet.addChild(markupContainer);

			var txtBox = new TextBox({
				id: "setMarkupTxtBox",
				value: at(self._setMarkupPopup.model, "reseller_markup"),
				pattern: consts.numberPattern,
				invalidMessage: consts.numberInvalidMessage,
				required: true
			});

			markupContainer.addChild(txtBox);

			markupContainer.addChild(new Output({
				id: "setMarkupPercentOutput",
				value: "%"
			}));

			self._setMarkupPopup.addChild(popupContainer);

			var page = registry.byId("pageContainer");
			page.addChild(self._setMarkupPopup);
		},

		_markupValid: function(){
			var page = registry.byId("pageContainer"),
				messages = page.get("messageList");
			messages.removeAll();
			return page.validate();
		},

		open: function(markup){
			var self = this;
			self._result = new Deferred();

			if (markup){
				self._setMarkupPopup.model.set("reseller_markup", markup);
			}

			self._setMarkupPopup.show();
			self._setMarkupPopup.resizePage();

			return self._result;
		}
	};

	return setMarkupPopup;
});