@ECHO off

REM Declare script variable

SET RESULT=0
SET INTERMEDIATEDIR=


REM Declare settings

SET SILENT=Yes
SET INSTANCENAME=
SET TARGETDIR=
SET DBLOGIN="sa"
SET DBPASSWORD=

REM There are no any settings passed from the command line: treat the installation as a full
REM interactive and don't close the console window on the exit.
IF [%1]==[] SET SILENT=No

REM Read command line arguments
:While1
IF [%1]==[] GOTO :EndWhile1
	SET name=%~1

	SET list1="SILENT""INSTANCENAME"
	CALL SET check1=%%list1:"%name%"=%%

	SET list2="TARGETDIR""DBLOGIN""DBPASSWORD"
	CALL SET check2=%%list2:"%name%"=%%

	IF NOT [%list1%]==[%check1%] (
		CALL SET %%name%%=%%~2
	) ELSE IF NOT [%list2%]==[%check2%] (
		SET %name%="%~2"
	) ELSE (
		REM Unknown setting
		ECHO Error: Setting "%name%" is unknown
		SET RESULT=-1
		GOTO :STOP
	)
	SHIFT /1
	SHIFT /1
	GOTO :While1
:EndWhile1

SET DEFAULT_TARGETDIR=%ProgramFiles%\Odin\Azure CSP

IF NOT [%INSTANCENAME%]==[] (
	SET DEFAULT_TARGETDIR=%DEFAULT_TARGETDIR% ^(%INSTANCENAME%^)
)

IF [%TARGETDIR%]==[] (
	REM Use CALL SET with double % to prevent a substituation of ')' symbol into the script code 
	SET TARGETDIR="%DEFAULT_TARGETDIR%"
)

CALL :Initialize %TARGETDIR%
SET RESULT=%ERRORLEVEL%
IF NOT [%RESULT%]==[0] GOTO :STOP

CALL :DeployFiles %TARGETDIR%
SET RESULT=%ERRORLEVEL%
IF NOT [%RESULT%]==[0] GOTO :STOP

CALL :RunUpdate %TARGETDIR% %INTERMEDIATEDIR% %DBLOGIN% %DBPASSWORD%
SET RESULT=%ERRORLEVEL%
IF NOT [%RESULT%]==[0] GOTO :STOP

:STOP
IF EXIST %INTERMEDIATEDIR% RMDIR /S /Q %INTERMEDIATEDIR%

IF /I [%SILENT%]==[No] PAUSE
EXIT /B %RESULT%


REM PROCEDURE: Initialize(base_dir)
:Initialize
	SET base_dir=%~1
	SET result=0

	IF NOT EXIST "%base_dir%\deployment" (
		ECHO Error: Cannot locate an instance of the application at "%base_dir%". The installation is aborted.
		SET result=-2
		GOTO :ReturnInitialize
	)

	SET COMPLUS_ApplicationMigrationRuntimeActivationConfigPath=%base_dir%\deployment

:ReturnInitialize
EXIT /B %result%

REM PROCEDURE: DeployFiles(base_dir)
:DeployFiles
	SET base_dir=%~1
	SET intermediate=%~1\deployment\new
	SET result=0

	ECHO Unpack distributive into "%intermediate%"

	IF EXIST "%intermediate%" RMDIR /S /Q "%intermediate%"
	IF ERRORLEVEL 1 (
		SET result=%ERRORLEVEL%
		ECHO Error: Cannot clean up "%intermediate%". The installation is aborted.
		GOTO :ReturnDeployFiles
	)

	MKDIR "%intermediate%"
	IF ERRORLEVEL 1 (
		SET result=%ERRORLEVEL%
		ECHO Error: Cannot create "%intermediate%". The installation is aborted.
		GOTO :ReturnDeployFiles
	)

	%windir%\System32\WindowsPowerShell\v1.0\powershell.exe ^
		-ExecutionPolicy Bypass ^
		-NoLogo ^
		-Command "Add-Type -assembly 'system.io.compression.filesystem'; [io.compression.zipfile]::ExtractToDirectory('%~dp0data.zip', '%intermediate%')"
	IF ERRORLEVEL 1 (
		SET result=%ERRORLEVEL%
		ECHO Error: Cannot unpack distributive into "%intermediate%". The installation is aborted.
		GOTO :ReturnDeployFiles
	)

	SET INTERMEDIATEDIR="%intermediate%"
	:ReturnDeployFiles
EXIT /B %result%

REM PROCEDURE: RunUpdate(target_dir, intermediate_dir, db_login, db_password)
:RunUpdate
	SET target_dir=%~1
	SET intermediate_dir=%~2
	SET db_login=%~3
	SET db_password=%~4

	PUSHD "%intermediate_dir%\deployment"
	%windir%\System32\WindowsPowerShell\v1.0\powershell.exe ^
		-ExecutionPolicy Bypass ^
		-NoLogo ^
		-Command .\Update.ps1 ^
		-TargetDir '%target_dir%' ^
		-IntermediateDir '%intermediate_dir%' ^
		-DbLogin '%db_login%' ^
		-DbPassword '%db_password%'
	Rem Fix colorizer in Sublime"

	SET RESULT=%ERRORLEVEL%
	POPD
EXIT /B %result%
